# OV_assets_addasset.py

import bpy
from bpy.props import StringProperty
from bpy.types import PropertyGroup, Operator, Panel
import os

class OV_AssetsProperties(PropertyGroup):
    asset_name: StringProperty(
        name="Asset Name",
        description="Nom de l'asset",
        default=""
    )
    asset_preview: StringProperty(
        name="Preview Image",
        description="Chemin vers l'image de prévisualisation (sera redimensionnée en 1024x1024 et enregistrée en JPG)",
        subtype='FILE_PATH'
    )

class OV_ASSETS_OT_AddSelectedAsset(Operator):
    bl_idname = "ov_assets.add_selected_asset"
    bl_label = "Add Selected Asset"
    bl_description = "Ajouter l'objet sélectionné dans le viewport à la bibliothèque d'assets avec une image preview redimensionnée"

    def execute(self, context):
        asset_props = context.scene.ov_asset_props
        asset_name = asset_props.asset_name.strip()
        preview_path = asset_props.asset_preview.strip()
        
        # Vérifications de base
        if not asset_name:
            self.report({'ERROR'}, "Le nom de l'asset est requis")
            return {'CANCELLED'}
        if not preview_path or not os.path.exists(preview_path):
            self.report({'ERROR'}, "Un chemin valide pour l'image de prévisualisation est requis")
            return {'CANCELLED'}
        
        # Traitement de l'image avec Blender
        try:
            # Chargement de l'image depuis le chemin fourni
            img = bpy.data.images.load(preview_path)
        except Exception as e:
            self.report({'ERROR'}, f"Impossible de charger l'image: {e}")
            return {'CANCELLED'}
        
        # Redimensionnement à 1024x1024 si nécessaire
        width, height = img.size
        if width != 1024 or height != 1024:
            try:
                img.scale(1024, 1024)
            except Exception as e:
                self.report({'ERROR'}, f"Erreur lors du redimensionnement de l'image: {e}")
                return {'CANCELLED'}
        
        # Configuration du format et sauvegarde de l'image en JPEG
        try:
            img.file_format = 'JPEG'
            from .OV_assets_library import get_icons_path
            dest_icon_dir = get_icons_path()
            if not os.path.exists(dest_icon_dir):
                os.makedirs(dest_icon_dir)
            dest_icon_path = os.path.join(dest_icon_dir, asset_name + ".jpg")
            # Sauvegarde via save_render qui permet de définir un chemin de destination
            img.save_render(dest_icon_path)
        except Exception as e:
            self.report({'ERROR'}, f"Erreur lors de la sauvegarde de l'image preview: {e}")
            return {'CANCELLED'}
        
        # Récupération de l'objet actif dans le viewport
        obj = context.active_object
        if not obj:
            self.report({'ERROR'}, "Aucun objet actif sélectionné")
            return {'CANCELLED'}
        
        # Export de l'objet actif dans un fichier blend dans le dossier assets/blends
        from .OV_assets_library import get_blends_path, unload_previews, load_previews
        dest_blend_dir = get_blends_path()
        if not os.path.exists(dest_blend_dir):
            os.makedirs(dest_blend_dir)
        dest_blend_path = os.path.join(dest_blend_dir, asset_name + ".blend")
        
        try:
            # Exportation de l'objet dans un fichier blend (en sauvegardant uniquement cet objet)
            bpy.data.libraries.write(dest_blend_path, {obj}, compress=True)
        except Exception as e:
            self.report({'ERROR'}, f"Échec de l'export de l'asset vers le fichier blend : {e}")
            return {'CANCELLED'}
        
        # Mise à jour des previews dans la bibliothèque d'assets
        unload_previews()
        load_previews()
        
        self.report({'INFO'}, f"L'asset '{asset_name}' a été ajouté avec succès")
        return {'FINISHED'}

class OV_ASSETS_PT_AddAssetPanel(Panel):
    bl_label = "Add Asset"
    bl_idname = "OV_ASSETS_PT_AddAssetPanel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OV Assets'
    
    def draw(self, context):
        layout = self.layout
        asset_props = context.scene.ov_asset_props
        
        layout.prop(asset_props, "asset_name")
        layout.prop(asset_props, "asset_preview")
        layout.operator("ov_assets.add_selected_asset", text="Add Selected Asset")

def register():
    bpy.utils.register_class(OV_AssetsProperties)
    bpy.utils.register_class(OV_ASSETS_OT_AddSelectedAsset)
    bpy.utils.register_class(OV_ASSETS_PT_AddAssetPanel)
    print("Module Add Asset (OV_assets_addasset) enregistré.")

def unregister():
    bpy.utils.unregister_class(OV_ASSETS_PT_AddAssetPanel)
    bpy.utils.unregister_class(OV_ASSETS_OT_AddSelectedAsset)
    bpy.utils.unregister_class(OV_AssetsProperties)
    print("Module Add Asset (OV_assets_addasset) désenregistré.")
