# OV_assets_library.py

import bpy
import os
import zipfile
import bpy.utils.previews
import bpy_extras.view3d_utils as v3d_utils
from bpy.props import StringProperty
from bpy.types import Operator, Panel

# --- Fonctions utilitaires de chemins ---
def get_assets_root():
    addon_dir = os.path.dirname(__file__)
    return os.path.join(addon_dir, "assets")  # Ce dossier devra contenir vos sous-dossiers "icons" et "blends"

def get_icons_path():
    return os.path.join(get_assets_root(), "icons")

def get_blends_path():
    return os.path.join(get_assets_root(), "blends")

# --- Gestion des previews ---
preview_collections = {}

def load_previews():
    pcoll = bpy.utils.previews.new()
    icons_path = get_icons_path()
    try:
        files = os.listdir(icons_path)
    except Exception as e:
        files = []
    for filename in files:
        if filename.lower().endswith((".jpg", ".png")):
            asset_name = os.path.splitext(filename)[0]
            filepath = os.path.join(icons_path, filename)
            pcoll.load(asset_name, filepath, 'IMAGE')
    preview_collections["asset_lib"] = pcoll

def unload_previews():
    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)
    preview_collections.clear()

def update_asset_library_enum(self, context):
    items = []
    pcoll = preview_collections.get("asset_lib")
    if pcoll:
        for i, key in enumerate(sorted(pcoll.keys())):
            items.append((key, key, "", pcoll[key].icon_id, i))
    if not items:
        items.append(("NONE", "None", "", 'QUESTION', 0))
    return items

def update_assetlib(self, context):
    # Callback vide pour rafraîchir l'interface si nécessaire
    pass

# --- Fonction d'append d'un asset ---
def append_asset(asset_name, blend_filepath):
    """
    Append (importe) l'asset dont le nom est donné depuis le fichier blend spécifié.
    Retourne l'objet asset ou None en cas d'erreur.
    """
    if asset_name in bpy.data.objects:
        return bpy.data.objects[asset_name]
    
    directory = blend_filepath + "/Object/"
    filepath = directory + asset_name

    try:
        bpy.ops.wm.append(
            filepath=filepath,
            directory=directory,
            filename=asset_name
        )
        print(f"Appended asset '{asset_name}' from '{blend_filepath}'")
    except Exception as e:
        print(f"Error appending asset '{asset_name}' from '{blend_filepath}': {e}")
        return None

    return bpy.data.objects.get(asset_name)

# --- Opérateurs de la Library ---

class OV_ASSETS_OT_AddLibrary(Operator):
    bl_idname = "ov_assets.add_library"
    bl_label = "Add Library"
    bl_description = "Import a zip file containing asset previews and blend files"
    
    filepath: StringProperty(subtype="FILE_PATH")
    
    def execute(self, context):
        if not self.filepath.lower().endswith(".zip"):
            self.report({'ERROR'}, "Please select a zip file")
            return {'CANCELLED'}
        try:
            with zipfile.ZipFile(self.filepath, 'r') as zf:
                for info in zf.infolist():
                    if info.is_dir():
                        continue
                    filename = os.path.basename(info.filename)
                    if info.filename.lower().endswith((".jpg", ".png")):
                        dest = os.path.join(get_icons_path(), filename)
                        with open(dest, "wb") as f:
                            f.write(zf.read(info.filename))
                    elif info.filename.lower().endswith(".blend"):
                        dest = os.path.join(get_blends_path(), filename)
                        with open(dest, "wb") as f:
                            f.write(zf.read(info.filename))
            self.report({'INFO'}, "Library imported successfully")
        except Exception as e:
            self.report({'ERROR'}, f"Failed to import library: {e}")
            return {'CANCELLED'}
        unload_previews()
        load_previews()
        return {'FINISHED'}
    
    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

class OV_ASSETS_OT_ExportLibrary(Operator):
    bl_idname = "ov_assets.export_library"
    bl_label = "Export Library"
    bl_description = "Export the entire asset library as a zip file with icons and blends folders"
    
    filepath: StringProperty(subtype="FILE_PATH")
    
    def execute(self, context):
        if not self.filepath.lower().endswith(".zip"):
            self.filepath += ".zip"
        try:
            with zipfile.ZipFile(self.filepath, 'w', zipfile.ZIP_DEFLATED) as zf:
                for foldername, subfolders, filenames in os.walk(get_icons_path()):
                    for filename in filenames:
                        file_path = os.path.join(foldername, filename)
                        arcname = os.path.join("icons", os.path.basename(file_path))
                        zf.write(file_path, arcname)
                for foldername, subfolders, filenames in os.walk(get_blends_path()):
                    for filename in filenames:
                        file_path = os.path.join(foldername, filename)
                        arcname = os.path.join("blends", os.path.basename(file_path))
                        zf.write(file_path, arcname)
            self.report({'INFO'}, "Library exported successfully")
        except Exception as e:
            self.report({'ERROR'}, f"Failed to export library: {e}")
            return {'CANCELLED'}
        return {'FINISHED'}
    
    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

class OV_ASSETS_OT_DeleteLibraryAsset(Operator):
    bl_idname = "ov_assets.delete_library_asset"
    bl_label = "Delete Library Asset"
    bl_description = "Delete the selected asset from the library after confirmation"
    
    def draw(self, context):
        layout = self.layout
        layout.label(text="Are you sure you want to delete this asset from your library?")
    
    def execute(self, context):
        asset_name = context.scene.ov_assetlib_selected
        if asset_name == "NONE":
            self.report({'WARNING'}, "No asset selected")
            return {'CANCELLED'}
        icon_file = os.path.join(get_icons_path(), asset_name + ".jpg")
        if not os.path.exists(icon_file):
            icon_file = os.path.join(get_icons_path(), asset_name + ".png")
        if os.path.exists(icon_file):
            try:
                os.remove(icon_file)
            except Exception as e:
                self.report({'ERROR'}, f"Failed to delete icon: {e}")
                return {'CANCELLED'}
        blend_file = os.path.join(get_blends_path(), asset_name + ".blend")
        if os.path.exists(blend_file):
            try:
                os.remove(blend_file)
            except Exception as e:
                self.report({'ERROR'}, f"Failed to delete blend file: {e}")
                return {'CANCELLED'}
        self.report({'INFO'}, f"Asset '{asset_name}' deleted from library")
        unload_previews()
        load_previews()
        return {'FINISHED'}
    
    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)

class OV_ASSETS_OT_ApplyLibraryAsset(Operator):
    bl_idname = "ov_assets.apply_library_asset"
    bl_label = "Apply Library Asset"
    bl_description = "Append the selected asset from the library to the scene"
    
    def execute(self, context):
        asset_name = context.scene.ov_assetlib_selected
        if asset_name == "NONE":
            self.report({'WARNING'}, "No asset selected")
            return {'CANCELLED'}
        asset_obj = bpy.data.objects.get(asset_name)
        if not asset_obj:
            # Tente d'append l'asset depuis son fichier blend
            blend_file = os.path.join(get_blends_path(), asset_name + ".blend")
            if not os.path.exists(blend_file):
                self.report({'ERROR'}, f"Blend file for asset '{asset_name}' not found")
                return {'CANCELLED'}
            asset_obj = append_asset(asset_name, blend_file)
            if not asset_obj:
                self.report({'ERROR'}, f"Failed to load asset '{asset_name}'")
                return {'CANCELLED'}
        # Lier l'asset à la scène s'il ne l'est pas déjà
        if asset_obj.name not in context.scene.collection.objects:
            context.scene.collection.objects.link(asset_obj)
        self.report({'INFO'}, f"Asset '{asset_name}' added to the scene.")
        return {'FINISHED'}

class OV_ASSETS_OT_RefreshPreviews(Operator):
    bl_idname = "ov_assets.refresh_previews"
    bl_label = "Refresh Previews"
    bl_description = "Refresh the asset previews from the icons folder"
    
    def execute(self, context):
        unload_previews()
        load_previews()
        pcoll = preview_collections.get("asset_lib")
        if pcoll and len(pcoll.keys()) > 0:
            context.scene.ov_assetlib_selected = list(pcoll.keys())[0]
        self.report({'INFO'}, "Previews refreshed")
        return {'FINISHED'}

# --- Panel de la Library ---
class OV_ASSETS_PT_LibraryPanel(Panel):
    bl_idname = "OV_ASSETS_PT_LibraryPanel"
    bl_label = "Asset Library"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OV Assets'
    
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        pcoll = preview_collections.get("asset_lib")
        if not pcoll:
            layout.label(text="No previews loaded")
        else:
            layout.label(text="Select an asset:")
            layout.template_icon_view(scene, "ov_assetlib_selected", scale=6, show_labels=True)
        row = layout.row(align=True)
        row.operator("ov_assets.refresh_previews", text="", icon='FILE_REFRESH')
        row.operator("ov_assets.apply_library_asset", text="Add Asset", icon='ADD')
        row.operator("ov_assets.delete_library_asset", text="", icon='TRASH')
        row = layout.row(align=True)
        row.operator("ov_assets.add_library", text="Add Library")
        row.operator("ov_assets.export_library", text="Export Library")

# --- Regroupement des classes à enregistrer ---
classes = (
    OV_ASSETS_OT_AddLibrary,
    OV_ASSETS_OT_ExportLibrary,
    OV_ASSETS_OT_DeleteLibraryAsset,
    OV_ASSETS_OT_ApplyLibraryAsset,
    OV_ASSETS_OT_RefreshPreviews,
    OV_ASSETS_PT_LibraryPanel,
)

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    print("Asset Library module registered.")

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    print("Asset Library module unregistered.")
