import bpy
import os
import shutil
import random
from bpy.props import (
    StringProperty, FloatProperty, FloatVectorProperty, BoolProperty, EnumProperty
)
from bpy.types import PropertyGroup, Operator, Panel

# Import des utilitaires depuis OV_materials_library.py
from .OV_materials_library import get_materials_root, get_icons_path, get_blends_path, append_node_group_from_blend

def generate_preview_from_color(color, out_filepath, width=1024, height=1024):
    image = bpy.data.images.new("temp_preview", width=width, height=height)
    pixels = [color[0], color[1], color[2], color[3]] * (width * height)
    image.pixels = pixels
    image.file_format = 'JPEG'
    image.filepath_raw = out_filepath
    image.save_render(out_filepath)
    bpy.data.images.remove(image)
    return True

def save_material_to_blend(mat, blend_filepath):
    try:
        bpy.data.libraries.write(blend_filepath, {mat}, compress=True)
        return True
    except Exception as e:
        print("Error saving material to blend:", e)
        return False

def get_existing_materials_enum(self, context):
    items = []
    for mat in bpy.data.materials:
        items.append((mat.name, mat.name, ""))
    if not items:
        items.append(("NONE", "None", ""))
    return items

# --- Property Group pour les paramètres du matériau ---
class OV_MaterialsProperties(PropertyGroup):
    mat_name: StringProperty(
        name="Material Name",
        description="Name of the new material",
        default="New_OV_Mat"
    )
    preview_image: StringProperty(
        name="Preview Image",
        description="File path for the preview image (will be converted to JPG 1024x1024)",
        subtype='FILE_PATH'
    )
    base_color_image: StringProperty(
        name="Base Color Image",
        description="File path for the base color image",
        subtype='FILE_PATH'
    )
    roughness_image: StringProperty(
        name="Roughness Image",
        description="File path for the roughness image",
        subtype='FILE_PATH'
    )
    normal_image: StringProperty(
        name="Normal Image",
        description="File path for the normal map image",
        subtype='FILE_PATH'
    )
    displacement_image: StringProperty(
        name="Displacement Image",
        description="File path for the displacement image",
        subtype='FILE_PATH'
    )
    # Propriétés pour le mode "Create PBR Material"
    ov_show_add_material: BoolProperty(
        name="Show Create Material",
        description="Toggle display of the Create Material panel",
        default=False
    )
    # Propriétés pour le mode "Save Material"
    existing_material: EnumProperty(
        name="Select Material",
        description="Select an existing material from the scene",
        items=get_existing_materials_enum
    )
    save_material_name: StringProperty(
        name="Material Name",
        description="Name for the saved material",
        default=""
    )
    save_preview_image: StringProperty(
        name="Preview Image",
        description="File path for the preview image (will be converted to JPG 1024x1024)",
        subtype='FILE_PATH'
    )
    ov_show_save_existing: BoolProperty(
        name="Show Save Material",
        description="Toggle display for saving a material",
        default=False
    )

# --- Opérateur : Créer et finaliser le matériau (Create PBR Material) ---
class OV_MATERIALS_OT_CreateMaterial(Operator):
    bl_idname = "ov_materials.create_material"
    bl_label = "Finalize Material"
    bl_description = "Finalize the creation of a new PBR material and add it to the library"
    
    def execute(self, context):
        props = context.scene.ov_material_props
        mat_name = props.mat_name.strip()
        if not mat_name:
            self.report({'ERROR'}, "Please provide a material name")
            return {'CANCELLED'}
        # Champs obligatoires
        if not props.preview_image:
            self.report({'ERROR'}, "Please provide a preview image")
            return {'CANCELLED'}
        if not props.base_color_image:
            self.report({'ERROR'}, "Please provide a base color image")
            return {'CANCELLED'}
        
        preview_source = bpy.path.abspath(props.preview_image)
        base_color_path = bpy.path.abspath(props.base_color_image)
        if not os.path.exists(preview_source):
            self.report({'ERROR'}, "Preview image file not found")
            return {'CANCELLED'}
        if not os.path.exists(base_color_path):
            self.report({'ERROR'}, "Base color image not found")
            return {'CANCELLED'}
        
        # Création du matériau et configuration du node tree
        mat = bpy.data.materials.new(name=mat_name)
        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        links = mat.node_tree.links
        nodes.clear()
        
        output_node = nodes.new(type="ShaderNodeOutputMaterial")
        output_node.location = (400, 0)
        bsdf_node = nodes.new(type="ShaderNodeBsdfPrincipled")
        bsdf_node.location = (0, 0)
        links.new(bsdf_node.outputs[0], output_node.inputs[0])
        
        # Attribution d'une couleur aléatoire pour la vue solide (Viewport Display)
        mat.diffuse_color = (random.random(), random.random(), random.random(), 1)
        
        # Base Color : image obligatoire
        base_tex_node = nodes.new(type="ShaderNodeTexImage")
        base_tex_node.location = (-400, 200)
        try:
            base_img = bpy.data.images.load(base_color_path)
            base_img.use_fake_user = True
            base_img.pack()
            base_tex_node.image = base_img
        except Exception as e:
            self.report({'ERROR'}, "Failed to load base color image")
            return {'CANCELLED'}
        links.new(base_tex_node.outputs['Color'], bsdf_node.inputs["Base Color"])
        
        # Utiliser l'image Preview pour générer la vignette
        preview_filename = mat_name + ".jpg"
        preview_filepath = os.path.join(get_icons_path(), preview_filename)
        try:
            img = bpy.data.images.load(preview_source)
            img.scale(1024, 1024)
            img.file_format = 'JPEG'
            img.filepath_raw = preview_filepath
            img.save_render(preview_filepath)
            bpy.data.images.remove(img)
        except Exception as e:
            self.report({'WARNING'}, f"Failed to process preview image: {e}")
        
        # Roughness : si une image est renseignée, on l'utilise, sinon on définit 0.2
        if props.roughness_image:
            roughness_path = bpy.path.abspath(props.roughness_image)
            if os.path.exists(roughness_path):
                rough_tex_node = nodes.new(type="ShaderNodeTexImage")
                rough_tex_node.location = (-400, 0)
                try:
                    rough_img = bpy.data.images.load(roughness_path)
                    rough_img.use_fake_user = True
                    rough_img.pack()
                    rough_img.colorspace_settings.name = 'Non-Color'
                    rough_tex_node.image = rough_img
                except Exception as e:
                    self.report({'ERROR'}, "Failed to load roughness image")
                    return {'CANCELLED'}
                links.new(rough_tex_node.outputs['Color'], bsdf_node.inputs["Roughness"])
            else:
                self.report({'WARNING'}, "Roughness image not found; using default value")
                bsdf_node.inputs["Roughness"].default_value = 0.2
        else:
            bsdf_node.inputs["Roughness"].default_value = 0.2
        
        # Normal Map : si renseignée
        normal_tex_node = None
        normal_map = None
        if props.normal_image:
            normal_path = bpy.path.abspath(props.normal_image)
            if os.path.exists(normal_path):
                normal_tex_node = nodes.new(type="ShaderNodeTexImage")
                normal_tex_node.location = (-400, -200)
                try:
                    normal_img = bpy.data.images.load(normal_path)
                    normal_img.use_fake_user = True
                    normal_img.pack()
                    normal_img.colorspace_settings.name = 'Non-Color'
                    normal_tex_node.image = normal_img
                except Exception as e:
                    self.report({'ERROR'}, "Failed to load normal image")
                normal_map = nodes.new(type="ShaderNodeNormalMap")
                normal_map.location = (-200, -200)
                links.new(normal_tex_node.outputs['Color'], normal_map.inputs['Color'])
                links.new(normal_map.outputs['Normal'], bsdf_node.inputs['Normal'])
        
        # Displacement via Bump Node : si renseignée
        if props.displacement_image:
            disp_path = bpy.path.abspath(props.displacement_image)
            if os.path.exists(disp_path):
                disp_tex_node = nodes.new(type="ShaderNodeTexImage")
                disp_tex_node.location = (-400, -400)
                try:
                    disp_img = bpy.data.images.load(disp_path)
                    disp_img.use_fake_user = True
                    disp_img.pack()
                    disp_img.colorspace_settings.name = 'Non-Color'
                    disp_tex_node.image = disp_img
                except Exception as e:
                    self.report({'ERROR'}, "Failed to load displacement image")
                    return {'CANCELLED'}
                bump_node = nodes.new(type="ShaderNodeBump")
                bump_node.location = (-200, -400)
                bump_node.inputs['Strength'].default_value = 0.1
                links.new(disp_tex_node.outputs['Color'], bump_node.inputs['Height'])
                if normal_map is not None:
                    links.new(normal_map.outputs['Normal'], bump_node.inputs['Normal'])
                links.new(bump_node.outputs['Normal'], bsdf_node.inputs['Normal'])
        
        # Append et ajout du node group "Textures preferencies"
        group_name = "Textures preferencies"
        ov_mat_blend = os.path.join(get_materials_root(), "OV_materials.blend")
        node_group = append_node_group_from_blend(group_name, ov_mat_blend)
        if node_group:
            tex_pref_node = nodes.new("ShaderNodeGroup")
            tex_pref_node.node_tree = node_group
            tex_pref_node.location = (-800, 150)
            if "Vector" in tex_pref_node.outputs:
                vector_output = tex_pref_node.outputs["Vector"]
                if base_tex_node and "Vector" in base_tex_node.inputs:
                    links.new(vector_output, base_tex_node.inputs["Vector"])
                if props.roughness_image and rough_tex_node and "Vector" in rough_tex_node.inputs:
                    links.new(vector_output, rough_tex_node.inputs["Vector"])
                if props.normal_image and normal_tex_node and "Vector" in normal_tex_node.inputs:
                    links.new(vector_output, normal_tex_node.inputs["Vector"])
                if props.displacement_image and disp_tex_node and "Vector" in disp_tex_node.inputs:
                    links.new(vector_output, disp_tex_node.inputs["Vector"])
            else:
                self.report({'WARNING'}, f"No 'Vector' output found on '{group_name}' node group.")
        
        # Sauvegarde du matériau dans un fichier .blend
        blend_filename = mat_name + ".blend"
        blend_filepath = os.path.join(get_blends_path(), blend_filename)
        if not save_material_to_blend(mat, blend_filepath):
            self.report({'WARNING'}, "Failed to save material to blend file")
        
        self.report({'INFO'}, f"Material '{mat_name}' added to library successfully.")
        
        # Réinitialisation des champs du panneau
        props.mat_name = ""
        props.preview_image = ""
        props.base_color_image = ""
        props.roughness_image = ""
        props.normal_image = ""
        props.displacement_image = ""
        props.ov_show_add_material = False
        
        return {'FINISHED'}

# --- Opérateur : Sauvegarder un matériau existant (Save Material) ---
class OV_MATERIALS_OT_SaveExistingMaterial(Operator):
    bl_idname = "ov_materials.save_existing_material"
    bl_label = "Save Material"
    bl_description = "Save an existing material from the scene to the library with a new name and preview image"
    
    def execute(self, context):
        props = context.scene.ov_material_props
        if props.existing_material == "NONE":
            self.report({'ERROR'}, "No material selected")
            return {'CANCELLED'}
        if props.save_material_name.strip() == "":
            self.report({'ERROR'}, "Please provide a new material name")
            return {'CANCELLED'}
        if not props.save_preview_image:
            self.report({'ERROR'}, "Please provide a preview image for saving the material")
            return {'CANCELLED'}
        preview_source = bpy.path.abspath(props.save_preview_image)
        if not os.path.exists(preview_source):
            self.report({'ERROR'}, "Preview image file not found")
            return {'CANCELLED'}
        
        existing = bpy.data.materials.get(props.existing_material)
        if not existing:
            self.report({'ERROR'}, f"Material '{props.existing_material}' not found")
            return {'CANCELLED'}
        new_mat = existing.copy()
        new_mat.name = props.save_material_name.strip()
        blend_filename = new_mat.name + ".blend"
        blend_filepath = os.path.join(get_blends_path(), blend_filename)
        if not save_material_to_blend(new_mat, blend_filepath):
            self.report({'WARNING'}, "Failed to save material to blend file")
        preview_filepath = os.path.join(get_icons_path(), new_mat.name + ".jpg")
        try:
            img = bpy.data.images.load(preview_source)
            img.scale(1024, 1024)
            img.file_format = 'JPEG'
            img.filepath_raw = preview_filepath
            img.save_render(preview_filepath)
            bpy.data.images.remove(img)
        except Exception as e:
            self.report({'WARNING'}, f"Failed to process preview image: {e}")
        self.report({'INFO'}, f"Material '{new_mat.name}' saved to library successfully.")
        props.ov_show_save_existing = False
        return {'FINISHED'}

# --- Opérateur : Afficher/Masquer le panneau Save Material ---
class OV_MATERIALS_OT_ToggleSaveExisting(Operator):
    bl_idname = "ov_materials.toggle_save_existing"
    bl_label = "Toggle Save Material Panel"
    bl_description = "Show or hide the Save Material panel"
    
    def execute(self, context):
        props = context.scene.ov_material_props
        props.ov_show_save_existing = not props.ov_show_save_existing
        return {'FINISHED'}

# --- Opérateur : Afficher/Masquer le panneau Create Material ---
class OV_MATERIALS_OT_ToggleAddMaterial(Operator):
    bl_idname = "ov_materials.toggle_add_material"
    bl_label = "Toggle Create Material Panel"
    bl_description = "Show or hide the Create PBR Material panel"
    
    def execute(self, context):
        props = context.scene.ov_material_props
        props.ov_show_add_material = not props.ov_show_add_material
        return {'FINISHED'}

# --- Panel de création / sauvegarde de matériau ---
class OV_MATERIALS_PT_AddMaterialPanel(Panel):
    bl_idname = "OV_MATERIALS_PT_AddMaterialPanel"
    bl_label = "Add material"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OV Materials'
    
    def draw(self, context):
        layout = self.layout
        props = context.scene.ov_material_props
        # Affichage des boutons initiaux l'un sur l'autre
        if not props.ov_show_add_material and not props.ov_show_save_existing:
            layout.operator("ov_materials.toggle_add_material", text="Create PBR Material", icon='TEXTURE')
            layout.operator("ov_materials.toggle_save_existing", text="Save Material", icon='FILE_TICK')
        # Panneau de création de matériau
        if props.ov_show_add_material:
            box = layout.box()
            box.label(text="Create PBR Material")
            box.label(text="Material Name:")
            box.prop(props, "mat_name", text="")
            box.label(text="Preview Image:")
            box.prop(props, "preview_image", text="")
            box.label(text="Base Color:")
            box.prop(props, "base_color_image", text="")
            box.label(text="Roughness:")
            box.prop(props, "roughness_image", text="")
            box.label(text="Normal Map:")
            box.prop(props, "normal_image", text="")
            box.label(text="Displacement:")
            box.prop(props, "displacement_image", text="")
            row = box.row(align=True)
            row.operator("ov_materials.create_material", text="Finalize Material", icon='CHECKMARK')
            row.operator("ov_materials.toggle_add_material", text="Close", icon='X')
        # Panneau de sauvegarde de matériau
        if props.ov_show_save_existing:
            box = layout.box()
            box.label(text="Save Material")
            box.label(text="Select Material:")
            box.prop(props, "existing_material", text="")
            box.label(text="Material Name:")
            box.prop(props, "save_material_name", text="")
            box.label(text="Preview Image:")
            box.prop(props, "save_preview_image", text="")
            row = box.row(align=True)
            row.operator("ov_materials.save_existing_material", text="Save Material", icon='CHECKMARK')
            row.operator("ov_materials.toggle_save_existing", text="Close", icon='X')

# -------------------------------------------------------------------
# Regroupement des classes à enregistrer
# -------------------------------------------------------------------
classes = (
    OV_MaterialsProperties,
    OV_MATERIALS_OT_CreateMaterial,
    OV_MATERIALS_OT_SaveExistingMaterial,
    OV_MATERIALS_OT_ToggleSaveExisting,
    OV_MATERIALS_OT_ToggleAddMaterial,
    OV_MATERIALS_PT_AddMaterialPanel,
)

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    print("Add Material module registered.")

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    print("Add Material module unregistered.")

if __name__ == "__main__":
    register()
