import bpy
import os
import zipfile
import shutil
import bpy.utils.previews
import bpy_extras.view3d_utils as v3d_utils
from bpy.props import StringProperty
from bpy.types import Operator, Panel

# --- Fonctions utilitaires de chemins ---
def get_materials_root():
    addon_dir = os.path.dirname(__file__)
    return os.path.join(addon_dir, "materials")

def get_icons_path():
    return os.path.join(get_materials_root(), "icons")

def get_blends_path():
    return os.path.join(get_materials_root(), "blends")

# --- Gestion des previews ---
preview_collections = {}

def load_previews():
    pcoll = bpy.utils.previews.new()
    icons_path = get_icons_path()
    try:
        files = os.listdir(icons_path)
    except Exception as e:
        files = []
    for filename in files:
        if filename.lower().endswith(".jpg"):
            mat_name = os.path.splitext(filename)[0]
            filepath = os.path.join(icons_path, filename)
            pcoll.load(mat_name, filepath, 'IMAGE')
    preview_collections["material_lib"] = pcoll

def unload_previews():
    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)
    preview_collections.clear()

def update_material_library_enum(self, context):
    items = []
    pcoll = preview_collections.get("material_lib")
    if pcoll:
        for i, key in enumerate(sorted(pcoll.keys())):
            items.append((key, key, "", pcoll[key].icon_id, i))
    if not items:
        items.append(("NONE", "None", "", 'QUESTION', 0))
    return items

def update_matlib(self, context):
    # Callback vide pour rafraîchir l'interface si nécessaire
    pass

# --- Fonction d'append du node group ---
def append_node_group_from_blend(group_name, blend_filepath):
    """
    Appends (imports) the given node group from the specified blend file.
    Returns the node group (bpy.types.NodeTree) or None if it fails.
    """
    if group_name in bpy.data.node_groups:
        return bpy.data.node_groups[group_name]
    
    # Le chemin doit être sous la forme: "/chemin/vers/OV_materials.blend/NodeTree/"
    directory = blend_filepath + "/NodeTree/"
    filepath = directory + group_name

    try:
        bpy.ops.wm.append(
            filepath=filepath,
            directory=directory,
            filename=group_name
        )
        print(f"Appended node group '{group_name}' from '{blend_filepath}'")
    except Exception as e:
        print(f"Error appending node group '{group_name}' from '{blend_filepath}': {e}")
        return None

    return bpy.data.node_groups.get(group_name)

# --- Opérateurs de la Library ---

class OV_MATERIALS_OT_AddLibrary(Operator):
    bl_idname = "ov_materials.add_library"
    bl_label = "Add Library"
    bl_description = "Import a zip file containing material previews and blend files"
    
    filepath: StringProperty(subtype="FILE_PATH")
    
    def execute(self, context):
        if not self.filepath.lower().endswith(".zip"):
            self.report({'ERROR'}, "Please select a zip file")
            return {'CANCELLED'}
        try:
            with zipfile.ZipFile(self.filepath, 'r') as zf:
                for info in zf.infolist():
                    # On ignore les répertoires
                    if info.is_dir():
                        continue
                    filename = os.path.basename(info.filename)
                    if info.filename.lower().endswith(".jpg"):
                        dest = os.path.join(get_icons_path(), filename)
                        with open(dest, "wb") as f:
                            f.write(zf.read(info.filename))
                    elif info.filename.lower().endswith(".blend"):
                        dest = os.path.join(get_blends_path(), filename)
                        with open(dest, "wb") as f:
                            f.write(zf.read(info.filename))
            self.report({'INFO'}, "Library imported successfully")
        except Exception as e:
            self.report({'ERROR'}, f"Failed to import library: {e}")
            return {'CANCELLED'}
        unload_previews()
        load_previews()
        return {'FINISHED'}
    
    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

class OV_MATERIALS_OT_ExportLibrary(Operator):
    bl_idname = "ov_materials.export_library"
    bl_label = "Export Library"
    bl_description = "Export the entire material library as a zip file with icons and blends folders"
    
    filepath: StringProperty(subtype="FILE_PATH")
    
    def execute(self, context):
        if not self.filepath.lower().endswith(".zip"):
            self.filepath += ".zip"
        try:
            with zipfile.ZipFile(self.filepath, 'w', zipfile.ZIP_DEFLATED) as zf:
                for foldername, subfolders, filenames in os.walk(get_icons_path()):
                    for filename in filenames:
                        file_path = os.path.join(foldername, filename)
                        arcname = os.path.join("icons", os.path.basename(file_path))
                        zf.write(file_path, arcname)
                for foldername, subfolders, filenames in os.walk(get_blends_path()):
                    for filename in filenames:
                        file_path = os.path.join(foldername, filename)
                        arcname = os.path.join("blends", os.path.basename(file_path))
                        zf.write(file_path, arcname)
            self.report({'INFO'}, "Library exported successfully")
        except Exception as e:
            self.report({'ERROR'}, f"Failed to export library: {e}")
            return {'CANCELLED'}
        return {'FINISHED'}
    
    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

class OV_MATERIALS_OT_DeleteLibraryMaterial(Operator):
    bl_idname = "ov_materials.delete_library_material"
    bl_label = "Delete Library Material"
    bl_description = "Delete the selected material from the library after confirmation"
    
    def draw(self, context):
        layout = self.layout
        layout.label(text="Are you sure you want to delete this material from your library?")
    
    def execute(self, context):
        mat_name = context.scene.ov_matlib_selected
        if mat_name == "NONE":
            self.report({'WARNING'}, "No material selected")
            return {'CANCELLED'}
        icon_file = os.path.join(get_icons_path(), mat_name + ".jpg")
        if os.path.exists(icon_file):
            try:
                os.remove(icon_file)
            except Exception as e:
                self.report({'ERROR'}, f"Failed to delete icon: {e}")
                return {'CANCELLED'}
        blend_file = os.path.join(get_blends_path(), mat_name + ".blend")
        if os.path.exists(blend_file):
            try:
                os.remove(blend_file)
            except Exception as e:
                self.report({'ERROR'}, f"Failed to delete blend file: {e}")
                return {'CANCELLED'}
        # Ne supprimez pas le matériau de bpy.data.materials pour ne pas impacter la scène
        self.report({'INFO'}, f"Material '{mat_name}' deleted from library")
        unload_previews()
        load_previews()
        return {'FINISHED'}
    
    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)

def append_material(mat_name):
    if mat_name in bpy.data.materials:
        return bpy.data.materials[mat_name]
    blends_file = os.path.join(get_blends_path(), mat_name + ".blend")
    if not os.path.exists(blends_file):
        print(f"{blends_file} not found.")
        return None
    directory = os.path.join(blends_file, "Material")
    try:
        bpy.ops.wm.append(filepath=os.path.join(directory, mat_name),
                          directory=directory,
                          filename=mat_name)
    except Exception as e:
        print("Error appending material:", e)
        return None
    return bpy.data.materials.get(mat_name)

class OV_MATERIALS_OT_ApplyLibraryMaterial(Operator):
    bl_idname = "ov_materials.apply_library_material"
    bl_label = "Apply Library Material"
    bl_description = "Apply the selected material from the library to all selected mesh objects using the active material slot"
    
    def execute(self, context):
        mat_name = context.scene.ov_matlib_selected
        if mat_name == "NONE":
            self.report({'WARNING'}, "No material selected")
            return {'CANCELLED'}
        mat = bpy.data.materials.get(mat_name)
        if not mat:
            mat = append_material(mat_name)
            if not mat:
                self.report({'ERROR'}, f"Failed to load material '{mat_name}'")
                return {'CANCELLED'}
        
        applied_objs = []
        for obj in context.selected_objects:
            if obj.type == 'MESH':
                slot_index = obj.active_material_index
                if obj.data.materials:
                    if slot_index < len(obj.data.materials):
                        obj.data.materials[slot_index] = mat
                    else:
                        obj.data.materials.append(mat)
                else:
                    obj.data.materials.append(mat)
                applied_objs.append(obj.name)
        
        if not applied_objs:
            self.report({'WARNING'}, "No mesh object selected")
            return {'CANCELLED'}
        
        for area in context.screen.areas:
            if area.type == 'VIEW_3D':
                area.tag_redraw()
                
        self.report({'INFO'}, f"Material '{mat_name}' applied to: {', '.join(applied_objs)}")
        return {'FINISHED'}

class OV_MATERIALS_OT_BucketLibraryMaterial(Operator):
    bl_idname = "ov_materials.bucket_library_material"
    bl_label = "Bucket Library Material"
    bl_description = ("Use the bucket to apply the selected library material by clicking on an object.\n"
                      "Press X or Enter to finish bucket mode.")
    
    material_name: StringProperty()
    
    def modal(self, context, event):
        context.window.cursor_set('CROSSHAIR')
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            region = context.region
            rv3d = context.region_data
            coord = (event.mouse_region_x, event.mouse_region_y)
            view_vector = v3d_utils.region_2d_to_vector_3d(region, rv3d, coord)
            ray_origin = v3d_utils.region_2d_to_origin_3d(region, rv3d, coord)
            depsgraph = context.evaluated_depsgraph_get()
            result, location, normal, index, obj, matrix = context.scene.ray_cast(depsgraph, ray_origin, view_vector)
            if result and obj and obj.type == 'MESH':
                mat = bpy.data.materials.get(self.material_name)
                if not mat:
                    mat = append_material(self.material_name)
                    if not mat:
                        self.report({'ERROR'}, f"Material '{self.material_name}' not found and could not be imported")
                        return {'RUNNING_MODAL'}
                # Ici, on récupère l'indice de slot associé à la face cliquée via l'index de la face
                if obj.data.materials and index >= 0:
                    material_index = obj.data.polygons[index].material_index
                    if material_index < len(obj.data.materials):
                        obj.data.materials[material_index] = mat
                    else:
                        obj.data.materials.append(mat)
                else:
                    obj.data.materials.append(mat)
                self.report({'INFO'}, f"Material '{self.material_name}' applied to '{obj.name}'.")
                for area in context.screen.areas:
                    if area.type == 'VIEW_3D':
                        area.tag_redraw()
            return {'RUNNING_MODAL'}
        elif event.type in {'X', 'RET', 'NUMPAD_ENTER'} and event.value == 'PRESS':
            context.window.cursor_set('DEFAULT')
            self.report({'INFO'}, "Bucket mode finished")
            return {'FINISHED'}
        elif event.type in {'RIGHTMOUSE', 'ESC'}:
            context.window.cursor_set('DEFAULT')
            self.report({'INFO'}, "Bucket canceled")
            return {'CANCELLED'}
        return {'RUNNING_MODAL'}
    
    def invoke(self, context, event):
        self.material_name = context.scene.ov_matlib_selected
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

class OV_MATERIALS_OT_RefreshPreviews(Operator):
    bl_idname = "ov_materials.refresh_previews"
    bl_label = "Refresh Previews"
    bl_description = "Refresh the material previews from the icons folder"
    
    def execute(self, context):
        unload_previews()
        load_previews()
        pcoll = preview_collections.get("material_lib")
        if pcoll and len(pcoll.keys()) > 0:
            context.scene.ov_matlib_selected = list(pcoll.keys())[0]
        self.report({'INFO'}, "Previews refreshed")
        return {'FINISHED'}

# --- Panel de la Library ---
class OV_MATERIALS_PT_LibraryPanel(Panel):
    bl_idname = "OV_MATERIALS_PT_LibraryPanel"
    bl_label = "Material Library"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OV Materials'
    
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        pcoll = preview_collections.get("material_lib")
        if not pcoll:
            layout.label(text="No previews loaded")
        else:
            layout.label(text="Select a material:")
            layout.template_icon_view(scene, "ov_matlib_selected", scale=6, show_labels=True)
        row = layout.row(align=True)
        row.operator("ov_materials.refresh_previews", text="", icon='FILE_REFRESH')
        row.operator("ov_materials.apply_library_material", text="Apply", icon='CHECKMARK')
        row.operator("ov_materials.bucket_library_material", text="", icon='TRACKER')
        row.operator("ov_materials.delete_library_material", text="", icon='TRASH')
        row = layout.row(align=True)
        row.operator("ov_materials.add_library", text="Add Library")
        row.operator("ov_materials.export_library", text="Export Library")

# --- Regroupement des classes à enregistrer ---
classes = (
    OV_MATERIALS_OT_AddLibrary,
    OV_MATERIALS_OT_ExportLibrary,
    OV_MATERIALS_OT_DeleteLibraryMaterial,
    OV_MATERIALS_OT_ApplyLibraryMaterial,
    OV_MATERIALS_OT_BucketLibraryMaterial,
    OV_MATERIALS_OT_RefreshPreviews,
    OV_MATERIALS_PT_LibraryPanel,
)

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    print("Library module registered.")

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    print("Library module unregistered.")

if __name__ == "__main__":
    register()
