import bpy
import bpy_extras.view3d_utils as v3d_utils
from bpy.props import EnumProperty, FloatProperty, PointerProperty, IntProperty
from bpy.types import PropertyGroup, Panel, Operator

# -------------------------------------------------------------------
# Fonctions utilitaires
# -------------------------------------------------------------------
def get_texture_preferences_node(mat):
    """
    Retourne le node du matériau dont le node_tree correspond à
    bpy.data.node_groups["Textures preferencies"].
    """
    target_ng = bpy.data.node_groups.get("Textures preferencies")
    if not target_ng:
        return None
    if mat and mat.use_nodes:
        for node in mat.node_tree.nodes:
            if hasattr(node, "node_tree") and node.node_tree == target_ng:
                return node
    return None

def get_transform_material_items(self, context):
    """
    Retourne la liste des matériaux de la scène possédant un node
    basé sur le NodeGroup "Textures preferencies".
    """
    items = []
    for mat in bpy.data.materials:
        if mat.use_nodes and get_texture_preferences_node(mat) is not None:
            items.append((mat.name, mat.name, ""))
    if not items:
        items.append(("NONE", "None", ""))
    return items

def update_transform_material(self, context):
    """
    Charge les valeurs (Rotation, Scale, Location X et Location Y) du node 
    "Textures preferencies" du matériau sélectionné.
    """
    mat = bpy.data.materials.get(self.transform_material)
    if mat:
        node = get_texture_preferences_node(mat)
        if node:
            if "Rotation Z" in node.inputs:
                self.rotation = node.inputs["Rotation Z"].default_value
            if "Scale X" in node.inputs:
                self.scale = node.inputs["Scale X"].default_value  # Supposé égal à Scale Y
            if "Location X" in node.inputs:
                self.location_x = node.inputs["Location X"].default_value
            if "Location Y" in node.inputs:
                self.location_y = node.inputs["Location Y"].default_value

def update_rotation(self, context):
    """
    Met à jour l'input "Rotation Z" du node "Textures preferencies" 
    avec la valeur du champ "Rotation".
    """
    mat = bpy.data.materials.get(self.transform_material)
    if mat:
        node = get_texture_preferences_node(mat)
        if node and "Rotation Z" in node.inputs:
            node.inputs["Rotation Z"].default_value = self.rotation

def update_scale(self, context):
    """
    Met à jour les inputs "Scale X" et "Scale Y" du node "Textures preferencies" 
    avec la valeur du champ "Scale".
    """
    mat = bpy.data.materials.get(self.transform_material)
    if mat:
        node = get_texture_preferences_node(mat)
        if node:
            if "Scale X" in node.inputs:
                node.inputs["Scale X"].default_value = self.scale
            if "Scale Y" in node.inputs:
                node.inputs["Scale Y"].default_value = self.scale

def update_location_x(self, context):
    mat = bpy.data.materials.get(self.transform_material)
    if mat:
        node = get_texture_preferences_node(mat)
        if node and "Location X" in node.inputs:
            node.inputs["Location X"].default_value = self.location_x

def update_location_y(self, context):
    mat = bpy.data.materials.get(self.transform_material)
    if mat:
        node = get_texture_preferences_node(mat)
        if node and "Location Y" in node.inputs:
            node.inputs["Location Y"].default_value = self.location_y

# -------------------------------------------------------------------
# Property Group pour Transform Material Texture
# -------------------------------------------------------------------
class OV_TransformProperties(PropertyGroup):
    transform_material: EnumProperty(
        name="Scene Material",
        description="Select a material from the scene (must have a 'Textures preferencies' node)",
        items=get_transform_material_items,
        update=update_transform_material
    )
    rotation: FloatProperty(
        name="Rotation",
        description="Rotation (applied to 'Rotation Z' in the node)",
        default=0.0,
        update=update_rotation
    )
    scale: FloatProperty(
        name="Scale",
        description="Scale (applied to both 'Scale X' and 'Scale Y')",
        default=1.0,
        update=update_scale
    )
    location_x: FloatProperty(
        name="Location X",
        description="Location X",
        default=0.0,
        update=update_location_x
    )
    location_y: FloatProperty(
        name="Location Y",
        description="Location Y",
        default=0.0,
        update=update_location_y
    )
    toggle_state: IntProperty(
        name="Toggle State",
        default=0,
        min=0,
        max=2
    )

# -------------------------------------------------------------------
# Opérateur : Pick material using eyedropper
# -------------------------------------------------------------------
class OV_MATERIALS_OT_PickTransformMaterial(Operator):
    bl_idname = "ov_materials.pick_transform_material"
    bl_label = "Pick Transform Material"
    bl_description = "Select material by clicking on an object in the viewport"
    
    def modal(self, context, event):
        context.window.cursor_set('EYEDROPPER')
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            region = context.region
            rv3d = context.region_data
            coord = (event.mouse_region_x, event.mouse_region_y)
            view_vector = v3d_utils.region_2d_to_vector_3d(region, rv3d, coord)
            ray_origin = v3d_utils.region_2d_to_origin_3d(region, rv3d, coord)
            depsgraph = context.evaluated_depsgraph_get()
            result, location, normal, index, obj, matrix = context.scene.ray_cast(depsgraph, ray_origin, view_vector)
            context.window.cursor_set('DEFAULT')
            if result and obj and obj.type == 'MESH' and obj.data.materials:
                mat = obj.data.materials[0]
                if get_texture_preferences_node(mat) is None:
                    self.report({'WARNING'}, f"Material '{mat.name}' has no 'Textures preferencies' node.")
                    return {'CANCELLED'}
                context.scene.ov_transform_props.transform_material = mat.name
                self.report({'INFO'}, f"Material '{mat.name}' selected.")
                return {'FINISHED'}
            else:
                self.report({'WARNING'}, "No material found on object.")
                return {'CANCELLED'}
        elif event.type in {'RIGHTMOUSE', 'ESC'}:
            context.window.cursor_set('DEFAULT')
            self.report({'INFO'}, "Pick cancelled")
            return {'CANCELLED'}
        return {'RUNNING_MODAL'}
    
    def invoke(self, context, event):
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

# -------------------------------------------------------------------
# Opérateur : Refresh material list (optionnel, ici retiré de l'interface)
# -------------------------------------------------------------------
class OV_MATERIALS_OT_RefreshTransformMaterials(Operator):
    bl_idname = "ov_materials.refresh_transform_materials"
    bl_label = "Refresh Transform Materials"
    bl_description = "Refresh the list of scene materials with a 'Textures preferencies' node"
    
    def execute(self, context):
        items = get_transform_material_items(None, context)
        if items and items[0][0] != "NONE":
            context.scene.ov_transform_props.transform_material = items[0][0]
        else:
            context.scene.ov_transform_props.transform_material = "NONE"
        self.report({'INFO'}, "Material list refreshed.")
        return {'FINISHED'}

# -------------------------------------------------------------------
# Opérateur : Duplicate material (icône DUPLICATE)
# -------------------------------------------------------------------
class OV_MATERIALS_OT_DuplicateTransformMaterial(Operator):
    bl_idname = "ov_materials.duplicate_transform_material"
    bl_label = "Duplicate Transform Material"
    bl_description = "Duplicate the selected material"
    
    def execute(self, context):
        props = context.scene.ov_transform_props
        mat_name = props.transform_material
        if mat_name == "NONE":
            self.report({'WARNING'}, "No material selected")
            return {'CANCELLED'}
        orig_mat = bpy.data.materials.get(mat_name)
        if not orig_mat:
            self.report({'ERROR'}, "Material not found")
            return {'CANCELLED'}
        new_mat = orig_mat.copy()
        new_mat.name = mat_name + "_dup"
        props.transform_material = new_mat.name
        self.report({'INFO'}, f"Material duplicated: {new_mat.name}")
        return {'FINISHED'}

# -------------------------------------------------------------------
# Opérateur : Apply material to object (icône TRACKER)
# -------------------------------------------------------------------
class OV_MATERIALS_OT_ApplyTransformMaterial(Operator):
    bl_idname = "ov_materials.apply_transform_material"
    bl_label = "Apply Transform Material"
    bl_description = "Apply the selected transform material to clicked objects; left-click to apply repeatedly, right-click or ESC to exit."
    
    def modal(self, context, event):
        context.window.cursor_set('CROSSHAIR')
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            region = context.region
            rv3d = context.region_data
            coord = (event.mouse_region_x, event.mouse_region_y)
            view_vector = v3d_utils.region_2d_to_vector_3d(region, rv3d, coord)
            ray_origin = v3d_utils.region_2d_to_origin_3d(region, rv3d, coord)
            depsgraph = context.evaluated_depsgraph_get()
            result, location, normal, index, obj, matrix = context.scene.ray_cast(depsgraph, ray_origin, view_vector)
            if result and obj and obj.type == 'MESH':
                mat = bpy.data.materials.get(context.scene.ov_transform_props.transform_material)
                if not mat:
                    self.report({'ERROR'}, "Selected material not found")
                else:
                    if obj.data.materials:
                        obj.data.materials[0] = mat
                    else:
                        obj.data.materials.append(mat)
                    self.report({'INFO'}, f"Material '{mat.name}' applied to '{obj.name}'.")
            else:
                self.report({'WARNING'}, "No valid object found")
            return {'RUNNING_MODAL'}
        elif event.type in {'RIGHTMOUSE', 'ESC'}:
            context.window.cursor_set('DEFAULT')
            self.report({'INFO'}, "Apply finished")
            return {'FINISHED'}
        return {'RUNNING_MODAL'}
    
    def invoke(self, context, event):
        context.window_manager.modal_handler_add(self)
        context.window.cursor_set('CROSSHAIR')
        return {'RUNNING_MODAL'}

# -------------------------------------------------------------------
# Opérateur : Toggle faces direction (icône LOOP_FORWARDS)
# -------------------------------------------------------------------
class OV_MATERIALS_OT_ToggleFacesDirection(Operator):
    bl_idname = "ov_materials.toggle_faces_direction"
    bl_label = "Toggle Faces Direction"
    bl_description = "Cycle the box rotation values between presets"
    
    def execute(self, context):
        props = context.scene.ov_transform_props
        mat = bpy.data.materials.get(props.transform_material)
        if not mat:
            self.report({'WARNING'}, "No material selected")
            return {'CANCELLED'}
        node = get_texture_preferences_node(mat)
        if not node:
            self.report({'WARNING'}, "Selected material has no 'Textures preferencies' node")
            return {'CANCELLED'}
        if props.toggle_state == 0:
            if "Box rotation X" in node.inputs:
                node.inputs["Box rotation X"].default_value = 90.0
            if "Box rotation Y" in node.inputs:
                node.inputs["Box rotation Y"].default_value = 0.0
            if "Box rotation Z" in node.inputs:
                node.inputs["Box rotation Z"].default_value = 0.0
        elif props.toggle_state == 1:
            if "Box rotation X" in node.inputs:
                node.inputs["Box rotation X"].default_value = 0.0
            if "Box rotation Y" in node.inputs:
                node.inputs["Box rotation Y"].default_value = 90.0
            if "Box rotation Z" in node.inputs:
                node.inputs["Box rotation Z"].default_value = 0.0
        elif props.toggle_state == 2:
            if "Box rotation X" in node.inputs:
                node.inputs["Box rotation X"].default_value = 0.0
            if "Box rotation Y" in node.inputs:
                node.inputs["Box rotation Y"].default_value = 0.0
            if "Box rotation Z" in node.inputs:
                node.inputs["Box rotation Z"].default_value = 0.0
        props.toggle_state = (props.toggle_state + 1) % 3
        self.report({'INFO'}, "Faces direction toggled.")
        return {'FINISHED'}

# -------------------------------------------------------------------
# Panneau : Transform Material Texture
# -------------------------------------------------------------------
class OV_MATERIALS_PT_TransformPanel(Panel):
    bl_idname = "OV_MATERIALS_PT_TransformPanel"
    bl_label = "Transform Material Texture"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OV Materials'
    
    def draw(self, context):
        layout = self.layout
        props = context.scene.ov_transform_props
        
        # Affiche la liste déroulante pour sélectionner le matériau
        layout.prop(props, "transform_material", text="Select material")
        
        # Rangée de boutons : Refresh est retiré, donc on affiche Pick, Duplicate et Apply
        row = layout.row(align=True)
        row.operator("ov_materials.pick_transform_material", text="Select material", icon='EYEDROPPER')
        row.operator("ov_materials.duplicate_transform_material", text="", icon='DUPLICATE')
        row.operator("ov_materials.apply_transform_material", text="", icon='TRACKER')
        
        layout.separator()
        layout.prop(props, "rotation", text="Rotation")
        layout.prop(props, "scale", text="Scale")
        layout.prop(props, "location_x", text="Location X")
        layout.prop(props, "location_y", text="Location Y")
        layout.operator("ov_materials.toggle_faces_direction", text="Toggle faces direction", icon='LOOP_FORWARDS')

# -------------------------------------------------------------------
# Registration / Unregistration du module Transform
# -------------------------------------------------------------------
def register():
    bpy.utils.register_class(OV_TransformProperties)
    bpy.types.Scene.ov_transform_props = PointerProperty(type=OV_TransformProperties)
    bpy.utils.register_class(OV_MATERIALS_OT_PickTransformMaterial)
    bpy.utils.register_class(OV_MATERIALS_OT_RefreshTransformMaterials)
    bpy.utils.register_class(OV_MATERIALS_OT_DuplicateTransformMaterial)
    bpy.utils.register_class(OV_MATERIALS_OT_ApplyTransformMaterial)
    bpy.utils.register_class(OV_MATERIALS_OT_ToggleFacesDirection)
    bpy.utils.register_class(OV_MATERIALS_PT_TransformPanel)
    print("OV_materials_transform module registered.")

def unregister():
    bpy.utils.unregister_class(OV_MATERIALS_PT_TransformPanel)
    bpy.utils.unregister_class(OV_MATERIALS_OT_ToggleFacesDirection)
    bpy.utils.unregister_class(OV_MATERIALS_OT_ApplyTransformMaterial)
    bpy.utils.unregister_class(OV_MATERIALS_OT_DuplicateTransformMaterial)
    bpy.utils.unregister_class(OV_MATERIALS_OT_RefreshTransformMaterials)
    bpy.utils.unregister_class(OV_MATERIALS_OT_PickTransformMaterial)
    del bpy.types.Scene.ov_transform_props
    bpy.utils.unregister_class(OV_TransformProperties)
    print("OV_materials_transform module unregistered.")

if __name__ == "__main__":
    register()
