# OV_setup_camera.py

import bpy, math

def update_camera_position(self, context):
    cam_obj = ensure_camera_exists(context)
    if cam_obj:
        cam_obj.location = context.scene.ov_cam_position

def update_camera_rotation(self, context):
    cam_obj = ensure_camera_exists(context)
    if cam_obj:
        user_rot = context.scene.ov_cam_rotation
        if context.scene.ov_align_vertical:
            user_angle_deg = math.degrees(user_rot[0])
            diff = user_angle_deg - 90.0
            cam_obj.rotation_euler.x = math.radians(90.0)
            cam_obj.rotation_euler.y = user_rot[1]
            cam_obj.rotation_euler.z = user_rot[2]
            cam_obj.data.shift_y = math.tan(math.radians(diff))
        else:
            cam_obj.rotation_euler = user_rot
            cam_obj.data.shift_y = 0.0

def update_camera_fov(self, context):
    cam_obj = ensure_camera_exists(context)
    if cam_obj:
        cam_obj.data.angle = math.radians(context.scene.ov_cam_fov)

def register_camera_properties():
    bpy.types.Scene.ov_cam_position = bpy.props.FloatVectorProperty(
        name="Position",
        subtype='XYZ',
        default=(0.0, 0.0, 1.5),
        description="Camera position (X, Y, Z)",
        update=update_camera_position
    )
    bpy.types.Scene.ov_cam_rotation = bpy.props.FloatVectorProperty(
        name="Rotation",
        subtype='EULER',
        unit='ROTATION',
        default=(math.radians(90.0), 0.0, 0.0),
        description="Camera rotation (in degrees)",
        update=update_camera_rotation
    )
    bpy.types.Scene.ov_align_vertical = bpy.props.BoolProperty(
        name="Align Vertical",
        description="Force vertical alignment",
        default=False,
        update=update_camera_rotation
    )
    bpy.types.Scene.ov_show_camera = bpy.props.BoolProperty(
        name="Show Camera Controls",
        description="Show or hide camera controls",
        default=True
    )
    bpy.types.Scene.ov_cam_fov = bpy.props.FloatProperty(
        name="Field of view",
        description="Camera field of view (in degrees)",
        default=66.0,
        min=1.0,
        max=179.0,
        update=update_camera_fov
    )

def unregister_camera_properties():
    del bpy.types.Scene.ov_cam_position
    del bpy.types.Scene.ov_cam_rotation
    del bpy.types.Scene.ov_align_vertical
    del bpy.types.Scene.ov_show_camera
    del bpy.types.Scene.ov_cam_fov

def ensure_camera_exists(context):
    scene = context.scene
    cam_obj = bpy.data.objects.get("OpenVisionCamera")
    if not cam_obj or cam_obj.type != 'CAMERA':
        cam_data = bpy.data.cameras.new(name="OpenVisionCamera")
        cam_obj = bpy.data.objects.new("OpenVisionCamera", cam_data)
        scene.collection.objects.link(cam_obj)
        scene.camera = cam_obj
        cam_obj.location = scene.ov_cam_position
        cam_obj.rotation_euler = scene.ov_cam_rotation
        cam_data.shift_y = 0.0
        cam_obj.data.angle = math.radians(scene.ov_cam_fov)
        print("Camera 'OpenVisionCamera' created with position", cam_obj.location)
    return cam_obj

# Operators et Panel pour la caméra

class OV_SETUP_OT_ToggleCameraExistence(bpy.types.Operator):
    bl_idname = "ov_setup.toggle_camera_existence"
    bl_label = "Toggle Camera Existence"
    bl_description = "Add the camera if it doesn't exist, otherwise delete it"
    
    def execute(self, context):
        if "OpenVisionCamera" in bpy.data.objects:
            bpy.data.objects.remove(bpy.data.objects["OpenVisionCamera"], do_unlink=True)
            self.report({'INFO'}, "Camera deleted.")
        else:
            ensure_camera_exists(context)
            self.report({'INFO'}, "Camera added.")
        return {'FINISHED'}

class OV_SETUP_OT_ToggleCamera(bpy.types.Operator):
    bl_idname = "ov_setup.toggle_camera"
    bl_label = "Toggle Camera Controls"
    bl_description = "Toggle display of camera controls"
    
    def execute(self, context):
        context.scene.ov_show_camera = not context.scene.ov_show_camera
        return {'FINISHED'}

class OV_SETUP_OT_ToggleShading(bpy.types.Operator):
    bl_idname = "ov_setup.toggle_shading"
    bl_label = "Toggle Shading Type"
    bl_description = "Toggle the viewport shading between 'Rendered' and 'Solid'"
    
    def execute(self, context):
        toggled = False
        for area in context.screen.areas:
            if area.type == 'VIEW_3D':
                space = area.spaces.active
                if space.shading.type == 'SOLID':
                    space.shading.type = 'RENDERED'
                    self.report({'INFO'}, "Viewport shading set to Rendered")
                else:
                    space.shading.type = 'SOLID'
                    self.report({'INFO'}, "Viewport shading set to Solid")
                toggled = True
        if not toggled:
            self.report({'WARNING'}, "No 3D Viewport found!")
        return {'FINISHED'}

class OV_SETUP_PT_CameraPanel(bpy.types.Panel):
    bl_idname = "OV_SETUP_PT_CameraPanel"
    bl_label = "Camera"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OpenVision'
    
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        if "OpenVisionCamera" in bpy.data.objects:
            layout.operator("ov_setup.toggle_camera_existence", text="Delete Camera")
        else:
            layout.operator("ov_setup.toggle_camera_existence", text="Add Camera")
        layout.operator("ov_setup.toggle_camera", text="Show/Hide Camera Controls")
        if scene.ov_show_camera:
            col = layout.column()
            col.prop(scene, "ov_cam_fov", text="Field of view")
            col.prop(scene, "ov_cam_position", text="Position")
            col.prop(scene, "ov_cam_rotation", text="Rotation")
            col.prop(scene, "ov_align_vertical", text="Align Vertical")
            col.operator("ov_setup.toggle_shading", text="Toggle Shading Type")

def register():
    bpy.utils.register_class(OV_SETUP_OT_ToggleCameraExistence)
    bpy.utils.register_class(OV_SETUP_OT_ToggleCamera)
    bpy.utils.register_class(OV_SETUP_OT_ToggleShading)
    bpy.utils.register_class(OV_SETUP_PT_CameraPanel)
    register_camera_properties()

def unregister():
    unregister_camera_properties()
    bpy.utils.unregister_class(OV_SETUP_PT_CameraPanel)
    bpy.utils.unregister_class(OV_SETUP_OT_ToggleShading)
    bpy.utils.unregister_class(OV_SETUP_OT_ToggleCamera)
    bpy.utils.unregister_class(OV_SETUP_OT_ToggleCameraExistence)
