import bpy

def update_export_quality(self, context):
    scene = context.scene
    quality = scene.ov_export_quality
    if quality == 'LOW':
        samples = 20
    elif quality == 'MEDIUM':
        samples = 150
    elif quality == 'HIGH':
        samples = 400
    else:
        samples = 150
    scene.cycles.samples = samples
    print(f"Quality updated: {quality} ({samples} samples)")

def update_export_resolution(self, context):
    scene = context.scene
    scene.render.resolution_x = scene.ov_export_res_x
    scene.render.resolution_y = scene.ov_export_res_y
    print(f"Resolution updated: {scene.render.resolution_x} x {scene.render.resolution_y}")

def update_export_transparent(self, context):
    scene = context.scene
    scene.render.film_transparent = scene.ov_export_transparent
    scene.cycles.film_transparent_glass = scene.ov_export_transparent
    print(f"Transparent background updated: {scene.ov_export_transparent}")

def configure_extra_passes_file_output():
    """
    Crée (ou met à jour) le node File Output dans le compositor pour qu'il possède
    exactement 6 file slots nommés selon la liste souhaitée et lui assigne le dossier d'export.
    """
    scene = bpy.context.scene
    scene.use_nodes = True
    node_tree = scene.node_tree
    nodes = node_tree.nodes

    # Supprimer le node Composite s'il existe
    composite_node = nodes.get("Composite")
    if composite_node is not None:
        nodes.remove(composite_node)

    # Rechercher ou créer le node File Output dédié
    file_output = nodes.get("Extra Passes File Output")
    if file_output is None:
        file_output = nodes.new(type="CompositorNodeOutputFile")
        file_output.name = "Extra Passes File Output"
        file_output.label = "Extra Passes File Output"

    # Liste des noms de slots désirés
    desired_slots = ["Render", "Object-ID", "Material-ID", "Depth", "Albedo", "Alpha"]

    default_count = len(file_output.file_slots)  # normalement 1
    target_count = len(desired_slots)

    if default_count < target_count:
        for i in range(target_count - default_count):
            file_output.file_slots.new(desired_slots[default_count + i])
    for i, slot in enumerate(file_output.file_slots):
        slot.path = desired_slots[i]

    # Utiliser le dossier d'export défini dans la propriété, par défaut "/tmp/"
    file_output.base_path = scene.ov_export_folder

    print("File Output node configured with extra passes:")
    for slot in file_output.file_slots:
        print("  ", slot.path)

def configure_color_ramp_stops(color_ramp_node):
    """
    Configure le Color Ramp avec 11 color stops aux positions et couleurs demandées.
    """
    stops = [
        (0.00, "#4C0000FF"),
        (0.10, "#004C00FF"),
        (0.20, "#00004CFF"),
        (0.30, "#7F0000FF"),
        (0.40, "#007F00FF"),
        (0.50, "#00007FFF"),
        (0.60, "#B30000FF"),
        (0.70, "#00B200FF"),
        (0.80, "#0000E5FF"),
        (0.90, "#E50000FF"),
        (1.00, "#00E500FF")
    ]
    
    def hex_to_rgba(hex_str):
        hex_str = hex_str.lstrip('#')
        r = int(hex_str[0:2], 16) / 255.0
        g = int(hex_str[2:4], 16) / 255.0
        b = int(hex_str[4:6], 16) / 255.0
        a = int(hex_str[6:8], 16) / 255.0
        return (r, g, b, a)
    
    cr = color_ramp_node.color_ramp
    if len(cr.elements) >= 2:
        cr.elements[0].position = stops[0][0]
        cr.elements[0].color = hex_to_rgba(stops[0][1])
        cr.elements[1].position = stops[-1][0]
        cr.elements[1].color = hex_to_rgba(stops[-1][1])
    for pos, col in stops[1:-1]:
        elem = cr.elements.new(pos)
        elem.position = pos
        elem.color = hex_to_rgba(col)
    print("Color Ramp stops configured.")

def configure_object_id_nodes():
    """
    Crée et connecte le réseau de nodes pour le pass Object-ID :
      Render Layers (IndexOB) -> Normalize -> ColorRamp -> File Output (entrée "Object-ID")
    """
    scene = bpy.context.scene
    node_tree = scene.node_tree
    nodes = node_tree.nodes
    links = node_tree.links

    render_layers = nodes.get("Render Layers")
    if render_layers is None:
        for node in nodes:
            if node.type == 'RLAYERS':
                render_layers = node
                break
    if render_layers is None:
        render_layers = nodes.new(type="CompositorNodeRLayers")
        render_layers.name = "Render Layers"
        render_layers.location = (-300, 300)

    normalize_node = nodes.get("Normalize_ObjectID")
    if normalize_node is None:
        normalize_node = nodes.new(type="CompositorNodeNormalize")
        normalize_node.name = "Normalize_ObjectID"
        normalize_node.location = (0, 300)

    color_ramp_node = nodes.get("ColorRamp_ObjectID")
    if color_ramp_node is None:
        color_ramp_node = nodes.new(type="CompositorNodeValToRGB")
        color_ramp_node.name = "ColorRamp_ObjectID"
        color_ramp_node.location = (300, 300)
        configure_color_ramp_stops(color_ramp_node)

    file_output = nodes.get("Extra Passes File Output")
    if file_output is None:
        print("File Output node not found for Object-ID setup.")
        return

    if "IndexOB" not in render_layers.outputs:
        print("Render Layers node does not have an output 'IndexOB'.")
        return

    for link in list(links):
        if (link.from_node == render_layers and link.from_socket.name == "IndexOB") or \
           (link.to_node == normalize_node) or \
           (link.to_node == color_ramp_node) or \
           (link.to_node == file_output and link.to_socket.name == "Object-ID"):
            links.remove(link)

    links.new(render_layers.outputs["IndexOB"], normalize_node.inputs[0])
    links.new(normalize_node.outputs[0], color_ramp_node.inputs["Fac"])
    if "Object-ID" in file_output.inputs:
        links.new(color_ramp_node.outputs["Image"], file_output.inputs["Object-ID"])
        print("Object-ID node network configured.")
    else:
        print("Le node File Output ne possède pas de socket 'Object-ID'.")

def configure_material_id_nodes():
    """
    Duplique le procédé pour le slot Material-ID :
      Render Layers (IndexMA) -> Normalize_MaterialID -> ColorRamp_MaterialID -> File Output (entrée "Material-ID")
    """
    scene = bpy.context.scene
    node_tree = scene.node_tree
    nodes = node_tree.nodes
    links = node_tree.links

    render_layers = nodes.get("Render Layers")
    if render_layers is None:
        for node in nodes:
            if node.type == 'RLAYERS':
                render_layers = node
                break
    if render_layers is None:
        render_layers = nodes.new(type="CompositorNodeRLayers")
        render_layers.name = "Render Layers"
        render_layers.location = (-300, 100)

    normalize_node = nodes.get("Normalize_MaterialID")
    if normalize_node is None:
        normalize_node = nodes.new(type="CompositorNodeNormalize")
        normalize_node.name = "Normalize_MaterialID"
        normalize_node.location = (0, 100)

    color_ramp_node = nodes.get("ColorRamp_MaterialID")
    if color_ramp_node is None:
        color_ramp_node = nodes.new(type="CompositorNodeValToRGB")
        color_ramp_node.name = "ColorRamp_MaterialID"
        color_ramp_node.location = (300, 100)
        configure_color_ramp_stops(color_ramp_node)

    file_output = nodes.get("Extra Passes File Output")
    if file_output is None:
        print("File Output node not found for Material-ID setup.")
        return

    if "IndexMA" not in render_layers.outputs:
        print("Render Layers node does not have an output 'IndexMA'.")
        return

    for link in list(links):
        if (link.from_node == render_layers and link.from_socket.name == "IndexMA") or \
           (link.to_node == normalize_node) or \
           (link.to_node == color_ramp_node) or \
           (link.to_node == file_output and link.to_socket.name == "Material-ID"):
            links.remove(link)

    links.new(render_layers.outputs["IndexMA"], normalize_node.inputs[0])
    links.new(normalize_node.outputs[0], color_ramp_node.inputs["Fac"])
    if "Material-ID" in file_output.inputs:
        links.new(color_ramp_node.outputs["Image"], file_output.inputs["Material-ID"])
        print("Material-ID node network configured.")
    else:
        print("Le node File Output ne possède pas de socket 'Material-ID'.")

def configure_standard_passes_nodes():
    """
    Configure les connexions directes pour les passes standards :
      - Render Layers "Image" -> File Output "Render"
      - Render Layers "Denoising Depth" -> Normalize_Depth -> File Output "Depth"
      - Render Layers "Denoising Albedo" -> File Output "Albedo"
      - Render Layers "Alpha" -> File Output "Alpha"
    """
    scene = bpy.context.scene
    node_tree = scene.node_tree
    nodes = node_tree.nodes
    links = node_tree.links

    render_layers = nodes.get("Render Layers")
    if render_layers is None:
        for node in nodes:
            if node.type == 'RLAYERS':
                render_layers = node
                break
    if render_layers is None:
        render_layers = nodes.new(type="CompositorNodeRLayers")
        render_layers.name = "Render Layers"
        render_layers.location = (-300, 0)

    file_output = nodes.get("Extra Passes File Output")
    if file_output is None:
        print("File Output node not found for standard passes.")
        return

    for link in list(links):
        if link.to_node == file_output and link.to_socket.name in {"Render", "Depth", "Albedo", "Alpha"}:
            links.remove(link)

    # Connexion : Render Layers "Image" -> File Output "Render"
    if "Image" in render_layers.outputs and len(file_output.inputs) > 0:
        links.new(render_layers.outputs["Image"], file_output.inputs[0])
        print("Connected Render Layers 'Image' to File Output 'Render'")
    else:
        print("Render Layers missing 'Image' output or File Output missing input 0.")

    # Denoising Depth via Normalize_Depth
    normalize_depth = nodes.get("Normalize_Depth")
    if normalize_depth is None:
        normalize_depth = nodes.new(type="CompositorNodeNormalize")
        normalize_depth.name = "Normalize_Depth"
        normalize_depth.location = (300, -100)
    for link in list(links):
        if link.from_node == render_layers and link.from_socket.name == "Denoising Depth":
            links.remove(link)
        if link.to_node == normalize_depth:
            links.remove(link)
    if "Denoising Depth" in render_layers.outputs:
        links.new(render_layers.outputs["Denoising Depth"], normalize_depth.inputs[0])
        print("Connected Render Layers 'Denoising Depth' to Normalize_Depth node")
    else:
        print("Render Layers missing 'Denoising Depth' output.")
    if "Depth" in file_output.inputs:
        links.new(normalize_depth.outputs[0], file_output.inputs["Depth"])
        print("Connected Normalize_Depth to File Output 'Depth'")
    else:
        print("File Output missing 'Depth' input.")

    # Connexion directe pour Denoising Albedo
    if "Denoising Albedo" in render_layers.outputs and "Albedo" in file_output.inputs:
        links.new(render_layers.outputs["Denoising Albedo"], file_output.inputs["Albedo"])
        print("Connected Render Layers 'Denoising Albedo' to File Output 'Albedo'")
    else:
        print("Missing 'Denoising Albedo' output or File Output missing 'Albedo' input.")

    # Connexion directe pour Alpha
    if "Alpha" in render_layers.outputs and "Alpha" in file_output.inputs:
        links.new(render_layers.outputs["Alpha"], file_output.inputs["Alpha"])
        print("Connected Render Layers 'Alpha' to File Output 'Alpha'")
    else:
        print("Missing 'Alpha' output or File Output missing 'Alpha' input.")

def update_export_extra_passes(self, context):
    view_layer = context.view_layer
    scene = context.scene
    if scene.ov_export_extra_passes:
        if scene.render.engine != 'CYCLES':
            scene.render.engine = 'CYCLES'
            print("Render engine switched to Cycles.")

        if hasattr(view_layer, 'cycles'):
            view_layer.cycles.denoising_store_passes = True
        view_layer.use_pass_object_index = True
        view_layer.use_pass_material_index = True
        print("Extra passes enabled: Denoising Data, Object Index, Material Index")

        scene.use_nodes = True

        configure_extra_passes_file_output()
        configure_object_id_nodes()
        configure_material_id_nodes()
        configure_standard_passes_nodes()
    else:
        if hasattr(view_layer, 'cycles'):
            view_layer.cycles.denoising_store_passes = False
        view_layer.use_pass_object_index = False
        view_layer.use_pass_material_index = False
        print("Extra passes disabled")
        if scene.use_nodes:
            node_tree = scene.node_tree
            nodes = node_tree.nodes
            file_output = nodes.get("Extra Passes File Output")
            if file_output is not None:
                nodes.remove(file_output)
            normalize_node = nodes.get("Normalize_ObjectID")
            if normalize_node is not None:
                nodes.remove(normalize_node)
            color_ramp_node = nodes.get("ColorRamp_ObjectID")
            if color_ramp_node is not None:
                nodes.remove(color_ramp_node)
            normalize_node_mat = nodes.get("Normalize_MaterialID")
            if normalize_node_mat is not None:
                nodes.remove(normalize_node_mat)
            color_ramp_node_mat = nodes.get("ColorRamp_MaterialID")
            if color_ramp_node_mat is not None:
                nodes.remove(color_ramp_node_mat)
            normalize_depth = nodes.get("Normalize_Depth")
            if normalize_depth is not None:
                nodes.remove(normalize_depth)
        scene.use_nodes = False

def register_export_properties():
    bpy.types.Scene.ov_export_quality = bpy.props.EnumProperty(
        name="Quality",
        description="Render quality",
        items=[
            ('LOW', "Low", "20 Render Samples"),
            ('MEDIUM', "Medium", "150 Render Samples"),
            ('HIGH', "High", "400 Render Samples")
        ],
        default='MEDIUM',
        update=update_export_quality
    )
    bpy.types.Scene.ov_export_res_x = bpy.props.IntProperty(
        name="Resolution X",
        description="Image width in pixels",
        default=1920,
        min=1,
        update=update_export_resolution
    )
    bpy.types.Scene.ov_export_res_y = bpy.props.IntProperty(
        name="Resolution Y",
        description="Image height in pixels",
        default=1080,
        min=1,
        update=update_export_resolution
    )
    bpy.types.Scene.ov_export_transparent = bpy.props.BoolProperty(
        name="Transparent background",
        description="Enable transparent background and glass in Render/Film",
        default=False,
        update=update_export_transparent
    )
    bpy.types.Scene.ov_export_extra_passes = bpy.props.BoolProperty(
        name="Export Object-ID, Material-ID, Depth, Albedo and Alpha",
        description="Enable extra passes and add a File Output node in the compositor",
        default=False,
        update=update_export_extra_passes
    )
    bpy.types.Scene.ov_export_folder = bpy.props.StringProperty(
        name="Export Folder",
        description="Destination folder for extra passes images",
        subtype='DIR_PATH',
        default="/tmp/"
    )

def unregister_export_properties():
    del bpy.types.Scene.ov_export_quality
    del bpy.types.Scene.ov_export_res_x
    del bpy.types.Scene.ov_export_res_y
    del bpy.types.Scene.ov_export_transparent
    del bpy.types.Scene.ov_export_extra_passes
    del bpy.types.Scene.ov_export_folder

class OV_SETUP_OT_RenderImage(bpy.types.Operator):
    bl_idname = "ov_setup.render_image"
    bl_label = "Render image"
    bl_description = "Render the image with the selected settings"
    
    def execute(self, context):
        objIndex = 0
        matIndex = 0
        for obj in bpy.data.objects:
            obj.pass_index = objIndex + 1
            objIndex += 1
        for mat in bpy.data.materials:
            mat.pass_index = matIndex + 1
            matIndex += 1

        scene = context.scene
        if scene.ov_export_extra_passes:
            node_tree = scene.node_tree
            nodes = node_tree.nodes
            file_output = nodes.get("Extra Passes File Output")
            if file_output:
                file_output.base_path = scene.ov_export_folder
                print("Updated File Output base_path:", scene.ov_export_folder)
        
        scene.render.resolution_x = scene.ov_export_res_x
        scene.render.resolution_y = scene.ov_export_res_y
        quality = scene.ov_export_quality
        if quality == 'LOW':
            samples = 20
        elif quality == 'MEDIUM':
            samples = 150
        elif quality == 'HIGH':
            samples = 400
        else:
            samples = 150
        scene.cycles.samples = samples
        self.report({'INFO'}, f"Rendering started (Samples: {samples}, Resolution: {scene.render.resolution_x}x{scene.render.resolution_y})")
        bpy.ops.render.render('INVOKE_DEFAULT', write_still=True)
        return {'FINISHED'}

class OV_SETUP_PT_ExportPanel(bpy.types.Panel):
    bl_idname = "OV_SETUP_PT_ExportPanel"
    bl_label = "Export"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OpenVision'
    
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        box = layout.box()
        box.label(text="Export Settings")
        box.prop(scene, "ov_export_quality", text="Quality")
        box.prop(scene, "ov_export_res_x", text="Resolution X")
        box.prop(scene, "ov_export_res_y", text="Resolution Y")
        box.prop(scene, "ov_export_transparent", text="Transparent background")
        box.prop(scene, "ov_export_extra_passes", text="Export Object-ID, Material-ID, Depth, Albedo and Alpha")
        if scene.ov_export_extra_passes:
            box.prop(scene, "ov_export_folder", text="Export Folder")
        layout.operator("ov_setup.render_image", text="Render image")

def register():
    bpy.utils.register_class(OV_SETUP_OT_RenderImage)
    bpy.utils.register_class(OV_SETUP_PT_ExportPanel)
    register_export_properties()
    print("OV_setup_export registered.")

def unregister():
    unregister_export_properties()
    bpy.utils.unregister_class(OV_SETUP_PT_ExportPanel)
    bpy.utils.unregister_class(OV_SETUP_OT_RenderImage)
    print("OV_setup_export unregistered.")

if __name__ == "__main__":
    register()
