# OV_setup_import.py

import bpy
from bpy_extras.io_utils import ImportHelper
from bpy.props import StringProperty
import os

# ----------------------
# Fonctions utilitaires
# ----------------------
def get_materials_root():
    addon_dir = os.path.dirname(__file__)
    return os.path.join(addon_dir, "materials")

def get_icons_path():
    return os.path.join(get_materials_root(), "icons")

def get_blends_path():
    return os.path.join(get_materials_root(), "blends")

def create_import_collection():
    # Créer une collection "Import_X" avec X incrémenté si nécessaire
    i = 1
    while f"Import_{i}" in bpy.data.collections:
        i += 1
    collection_name = f"Import_{i}"
    new_coll = bpy.data.collections.new(collection_name)
    bpy.context.scene.collection.children.link(new_coll)
    return new_coll

def clear_parent_keep_transform(objects):
    bpy.ops.object.select_all(action='DESELECT')
    for obj in objects:
        obj.select_set(True)
    bpy.context.view_layer.objects.active = objects[0]
    bpy.ops.object.parent_clear(type='CLEAR_KEEP_TRANSFORM')

def apply_transformations(objects):
    bpy.ops.object.select_all(action='DESELECT')
    for obj in objects:
        obj.select_set(True)
        bpy.context.view_layer.objects.active = obj
    bpy.ops.object.transform_apply(location=True, rotation=True, scale=True)

def set_origin_to_geometry(objects):
    bpy.ops.object.select_all(action='DESELECT')
    for obj in objects:
        obj.select_set(True)
        bpy.context.view_layer.objects.active = obj
    bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='BOUNDS')

def remove_empties_from_collection(coll):
    empties = [obj for obj in coll.objects if obj.type == 'EMPTY']
    for empty in empties:
        coll.objects.unlink(empty)
        bpy.data.objects.remove(empty, do_unlink=True)

# ----------------------
# Opérateur d'import Collada
# ----------------------
class OV_SETUP_OT_ImportCollada(bpy.types.Operator, ImportHelper):
    bl_idname = "ov_setup.import_collada"
    bl_label = "Import Collada (.dae)"
    bl_description = "Import a 3D file in Collada (.dae) format"
    filename_ext = ".dae"
    filter_glob: StringProperty(default="*.dae;*.DAE", options={'HIDDEN'})
    
    def execute(self, context):
        if self.filepath:
            # Sauvegarder la liste des objets existants avant import
            prev_objs = set(bpy.data.objects.keys())
            
            bpy.ops.wm.collada_import(filepath=self.filepath)
            self.report({'INFO'}, f"Imported file: {self.filepath}")
            
            # Récupérer tous les nouveaux objets importés
            all_imported = [obj for obj in bpy.data.objects if obj.name not in prev_objs]
            # Séparer les objets à garder et ceux à supprimer (caméras et lumières)
            imported_objects = [obj for obj in all_imported if obj.type not in {'CAMERA', 'LIGHT'}]
            to_remove = [obj for obj in all_imported if obj.type in {'CAMERA', 'LIGHT'}]
            for obj in to_remove:
                bpy.data.objects.remove(obj, do_unlink=True)
            
            if not imported_objects:
                self.report({'WARNING'}, "No new objects imported (after filtering cameras/lights).")
                return {'FINISHED'}
            
            # Créer et remplir la collection d'import
            new_coll = create_import_collection()
            for obj in imported_objects:
                for coll in obj.users_collection:
                    coll.objects.unlink(obj)
                new_coll.objects.link(obj)
            
            # 2. Clear parent (Clear Parent Keep Transformation)
            clear_parent_keep_transform(imported_objects)
            
            # 3. Supprimer les "Empty" inutiles
            remove_empties_from_collection(new_coll)
            
            # 4. Appliquer transformations (location, rotation, scale)
            apply_transformations(new_coll.objects)
            
            # 5. Déplacer l'origine de chaque objet au centre de sa géométrie
            set_origin_to_geometry(new_coll.objects)
            
        else:
            self.report({'WARNING'}, "No file selected.")
        return {'FINISHED'}

# ----------------------
# Opérateur d'import FBX
# ----------------------
class OV_SETUP_OT_ImportFBX(bpy.types.Operator, ImportHelper):
    bl_idname = "ov_setup.import_fbx"
    bl_label = "Import FBX (.fbx)"
    bl_description = "Import a 3D file in FBX (.fbx) format"
    filename_ext = ".fbx"
    filter_glob: StringProperty(default="*.fbx;*.FBX", options={'HIDDEN'})
    
    def execute(self, context):
        if self.filepath:
            # Sauvegarder la liste des objets existants avant import
            prev_objs = set(bpy.data.objects.keys())
            
            bpy.ops.import_scene.fbx(filepath=self.filepath)
            self.report({'INFO'}, f"Imported file: {self.filepath}")
            
            # Récupérer tous les objets nouvellement importés
            all_imported = [obj for obj in bpy.data.objects if obj.name not in prev_objs]
            # Séparer les objets à garder (non caméra et non lumière) et ceux à supprimer
            imported_objects = [obj for obj in all_imported if obj.type not in {'CAMERA', 'LIGHT'}]
            to_remove = [obj for obj in all_imported if obj.type in {'CAMERA', 'LIGHT'}]
            for obj in to_remove:
                bpy.data.objects.remove(obj, do_unlink=True)
            
            if not imported_objects:
                self.report({'WARNING'}, "No new objects imported (after filtering cameras/lights).")
                return {'FINISHED'}
            
            # Créer et remplir la collection d'import
            new_coll = create_import_collection()
            for obj in imported_objects:
                for coll in obj.users_collection:
                    coll.objects.unlink(obj)
                new_coll.objects.link(obj)
            
            # 2. Clear parent (Clear Parent Keep Transformation)
            clear_parent_keep_transform(imported_objects)
            
            # 3. Supprimer les "Empty" inutiles
            remove_empties_from_collection(new_coll)
            
            # 4. Appliquer transformations (location, rotation, scale)
            apply_transformations(new_coll.objects)
            
            # 5. Déplacer l'origine de chaque objet au centre de sa géométrie
            set_origin_to_geometry(new_coll.objects)
            
        else:
            self.report({'WARNING'}, "No file selected.")
        return {'FINISHED'}

# ----------------------
# Panneau d'import
# ----------------------
class OV_SETUP_PT_ImportPanel(bpy.types.Panel):
    bl_idname = "OV_SETUP_PT_ImportPanel"
    bl_label = "Import"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OpenVision'
    
    def draw(self, context):
        layout = self.layout
        box = layout.box()
        box.label(text="Import Files:")
        box.operator("ov_setup.import_collada", text="Import Collada (.dae)")
        box.operator("ov_setup.import_fbx", text="Import FBX (.fbx)")

# ----------------------
# Enregistrement
# ----------------------
def register():
    bpy.utils.register_class(OV_SETUP_OT_ImportCollada)
    bpy.utils.register_class(OV_SETUP_OT_ImportFBX)
    bpy.utils.register_class(OV_SETUP_PT_ImportPanel)

def unregister():
    bpy.utils.unregister_class(OV_SETUP_PT_ImportPanel)
    bpy.utils.unregister_class(OV_SETUP_OT_ImportFBX)
    bpy.utils.unregister_class(OV_SETUP_OT_ImportCollada)

if __name__ == "__main__":
    register()
