# OV_setup_sky.py

import bpy, math

def ensure_sky_world(context):
    scene = context.scene
    world = bpy.data.worlds.get("OpenVision_Sky")
    if not world:
        world = bpy.data.worlds.new("OpenVision_Sky")
        world.use_nodes = True
        nodes = world.node_tree.nodes
        links = world.node_tree.links
        nodes.clear()
        node_sky_tex = nodes.new(type="ShaderNodeTexSky")
        node_sky_tex.name = "SkyTex"
        node_sky_tex.location = (-400, 0)
        node_sky_tex.sky_type = 'NISHITA'
        node_background = nodes.new(type="ShaderNodeBackground")
        node_background.name = "SkyBG"
        node_background.location = (-200, 0)
        node_output = nodes.new(type="ShaderNodeOutputWorld")
        node_output.location = (0, 0)
        links.new(node_sky_tex.outputs[0], node_background.inputs[0])
        links.new(node_background.outputs[0], node_output.inputs[0])
        print("World 'OpenVision_Sky' created.")
    scene.world = world
    return world

def update_sky_settings(context):
    scene = context.scene
    if not scene.ov_sky_enable:
        return
    world = ensure_sky_world(context)
    nodes = world.node_tree.nodes
    node_sky_tex = nodes.get("SkyTex")
    node_background = nodes.get("SkyBG")
    if node_sky_tex and node_background:
        day = scene.ov_sky_day
        month = scene.ov_sky_month
        time_val = scene.ov_sky_time
        elev_offset = scene.ov_sky_sun_elevation
        north_offset = scene.ov_sky_north
        ma = 365.2422
        mb = 0.38025574
        mc = 23.25707359
        md = -1.395049065
        me = 0.374674726
        mf = -1.465605465
        latitude = 47.1324
        longitude = 7.2441
        lar = latitude * (math.pi / 180)
        kun = math.pi / 180
        hv = time_val
        k = math.pi * (1 - hv / 12)
        gb = 2556 + day + (month * 30.4 - 30.4)
        i = mc * math.sin(2 * math.pi * gb / ma + md) + me * math.sin(4 * math.pi * gb / ma + mf) + mb
        h = kun * i
        g = math.cos(k)
        f = math.cos(lar)
        e = math.cos(h)
        d = math.sin(lar)
        c = math.sin(h)
        b = c * d + e * f * g
        a = math.asin(b)
        p = math.pi * (1 - hv / 12)
        o = math.sin(p)
        n = math.cos(a)
        m = (e * g * d - c * f) / n
        l_val = e * o / n
        if m >= 0:
            j = math.asin(l_val)
        else:
            j = -math.pi - math.asin(l_val)
        sun_elev = a + elev_offset
        sun_rot = ((j - north_offset) * -1) + (180 * kun)
        node_sky_tex.sun_elevation = sun_elev
        node_sky_tex.sun_rotation = sun_rot
        node_sky_tex.sun_disc = True
        node_sky_tex.sun_size = math.radians(scene.ov_sky_size)
        node_sky_tex.sun_intensity = scene.ov_sky_intensity
        node_background.inputs[1].default_value = scene.ov_sky_strength

def update_sky_enable(self, context):
    scene = context.scene
    if scene.ov_sky_enable:
        ensure_sky_world(context)
        update_sky_settings(context)
    else:
        if bpy.data.worlds.get("OpenVision_Sky"):
            bpy.data.worlds.remove(bpy.data.worlds["OpenVision_Sky"])

def update_sky_property(self, context):
    if context.scene.ov_sky_enable:
        update_sky_settings(context)

def register_sky_properties():
    sc = bpy.types.Scene
    sc.ov_sky_enable = bpy.props.BoolProperty(
        name="Enable Sky",
        description="Enable the sky (remove via 'Delete Sky')",
        default=False,
        update=update_sky_enable
    )
    sc.ov_sky_day = bpy.props.IntProperty(
        name="Day",
        description="Day of the month (1 to 31)",
        min=1, max=31,
        default=21,
        update=update_sky_property
    )
    sc.ov_sky_month = bpy.props.IntProperty(
        name="Month",
        description="Month (1 to 12)",
        min=1, max=12,
        default=6,
        update=update_sky_property
    )
    sc.ov_sky_time = bpy.props.FloatProperty(
        name="Time",
        description="Time of day (e.g., 12.5 for half past noon)",
        min=0.05, max=23.95,
        default=10,
        update=update_sky_property
    )
    sc.ov_sky_strength = bpy.props.FloatProperty(
        name="Sky Strength",
        description="Sky intensity",
        min=0, max=10,
        default=1,
        update=update_sky_property
    )
    sc.ov_sky_size = bpy.props.FloatProperty(
        name="Sun size (°)",
        description="Sun disc size in degrees",
        min=0, max=100,
        default=0.545,
        update=update_sky_property
    )
    sc.ov_sky_intensity = bpy.props.FloatProperty(
        name="Sun intensity",
        description="Sun intensity",
        min=0, max=10,
        default=1,
        update=update_sky_property
    )
    sc.ov_sky_sun_elevation = bpy.props.FloatProperty(
        name="Sun elevation",
        description="Sun elevation correction (in degrees)",
        min=-90, max=90,
        default=0,
        unit="ROTATION",
        update=update_sky_property
    )
    sc.ov_sky_north = bpy.props.FloatProperty(
        name="North",
        description="North direction correction (in degrees)",
        min=-360, max=360,
        default=0,
        unit="ROTATION",
        update=update_sky_property
    )
    sc.ov_show_sky = bpy.props.BoolProperty(
        name="Show Sky Controls",
        description="Show or hide sky controls",
        default=True
    )

def unregister_sky_properties():
    sc = bpy.types.Scene
    del sc.ov_sky_enable
    del sc.ov_sky_day
    del sc.ov_sky_month
    del sc.ov_sky_time
    del sc.ov_sky_strength
    del sc.ov_sky_size
    del sc.ov_sky_intensity
    del sc.ov_sky_sun_elevation
    del sc.ov_sky_north
    del sc.ov_show_sky

# Operators et Panel pour le ciel

class OV_SETUP_OT_AddSky(bpy.types.Operator):
    bl_idname = "ov_setup.add_sky"
    bl_label = "Add Sky"
    bl_description = "Add the 'OpenVision_Sky' world and enable the Sky module"
    
    def execute(self, context):
        context.scene.ov_sky_enable = True
        ensure_sky_world(context)
        self.report({'INFO'}, "Sky added and enabled.")
        return {'FINISHED'}

class OV_SETUP_OT_DeleteSky(bpy.types.Operator):
    bl_idname = "ov_setup.delete_sky"
    bl_label = "Delete Sky"
    bl_description = "Delete the 'OpenVision_Sky' world and disable the Sky module"
    
    def execute(self, context):
        context.scene.ov_sky_enable = False
        update_sky_enable(self, context)
        self.report({'INFO'}, "Sky deleted.")
        return {'FINISHED'}

class OV_SETUP_OT_ToggleSky(bpy.types.Operator):
    bl_idname = "ov_setup.toggle_sky"
    bl_label = "Toggle Sky Controls"
    bl_description = "Toggle display of sky controls"
    
    def execute(self, context):
        context.scene.ov_show_sky = not context.scene.ov_show_sky
        return {'FINISHED'}

class OV_SETUP_PT_SkyPanel(bpy.types.Panel):
    bl_idname = "OV_SETUP_PT_SkyPanel"
    bl_label = "Sky"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'OpenVision'
    
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        box = layout.box()
        if not scene.ov_sky_enable:
            box.operator("ov_setup.add_sky", text="Add Sky")
        else:
            box.operator("ov_setup.delete_sky", text="Delete Sky")
            layout.operator("ov_setup.toggle_sky", text=("Hide Sky Controls" if scene.ov_show_sky else "Show Sky Controls"))
            if scene.ov_show_sky:
                sky_box = layout.box()
                sky_box.prop(scene, "ov_sky_day", text="Day")
                sky_box.prop(scene, "ov_sky_month", text="Month")
                sky_box.prop(scene, "ov_sky_time", text="Time")
                sky_box.prop(scene, "ov_sky_strength", text="Sky Strength")
                sky_box.prop(scene, "ov_sky_size", text="Sun size (°)")
                sky_box.prop(scene, "ov_sky_intensity", text="Sun intensity")
                sky_box.prop(scene, "ov_sky_sun_elevation", text="Sun elevation")
                sky_box.prop(scene, "ov_sky_north", text="North")

def register():
    bpy.utils.register_class(OV_SETUP_OT_AddSky)
    bpy.utils.register_class(OV_SETUP_OT_DeleteSky)
    bpy.utils.register_class(OV_SETUP_OT_ToggleSky)
    bpy.utils.register_class(OV_SETUP_PT_SkyPanel)
    register_sky_properties()

def unregister():
    unregister_sky_properties()
    bpy.utils.unregister_class(OV_SETUP_PT_SkyPanel)
    bpy.utils.unregister_class(OV_SETUP_OT_ToggleSky)
    bpy.utils.unregister_class(OV_SETUP_OT_DeleteSky)
    bpy.utils.unregister_class(OV_SETUP_OT_AddSky)
